# coding: utf-8
import requests
from bs4 import BeautifulSoup as bs
from itertools import chain
import random
import pickle
from pprint import pprint
from os.path import exists

mirror_table = 'mirror_table.p'
allanimes = 'allanimes.p'
reject_list = ['facebook', 'bit.ly']


def get_soup(url):
    """
    returns BeautifulSoup(html content of url)
    """
    r = requests.get(url)
    if r.status_code == 200:
        return bs(r.content)
    else:
        return False

# results > div:nth-child(10) > a:nth-child(1)


def make_pickle():
    """
    make pickle file of all animes (for sampling)
    """
    url = 'http://www.nwanime.com/categories/'
    soup = get_soup(url)
    tuples = [(anchor.text, anchor.get('href'))
              for anchor in soup.select('#results > div > a:nth-of-type(1)')]
    with open('allanimes.p', 'w') as f:
        pickle.dump(tuples, f)


def get_animes_sample():
    """
    fetches a list of 10 random animes from pickle (for sampling)
    Note the pickle is generated by running this file's main()
    """
    f = open(allanimes)
    animes = pickle.load(f)
    return random.sample(animes, 10)


def get_random_episode(url):
    """
    Fetches a random episode from an anime page (for sampling)
    """
    soup = get_soup(url)
    try:
        episodes = [anchor.get('href') for anchor in soup.select(
            '#resultstats_large > div > a') if anchor.text.strip()]
        episode = random.choice(episodes)
    except:
        episode = None
    return episode


def get_mirrors(url):
    """
    returns a list of (mirror name,mirror link) tuples for given url
    """
    soup = get_soup(url)
    try:
        mirrors = [(anchor.text.split()[0], anchor.get('href'))
                   for anchor in soup.select('#video_mirrors > div > span.link > a') if '[US only]' not in anchor.text]
    except:
        mirrors = None
    return mirrors


class Table(object):
    """
    accumulator for sampler
    """

    def __init__(self):
        self.table = self.get_table()

    def get_table(self):
        if exists(mirror_table):
            with open(mirror_table) as f:
                table = pickle.load(f)
        else:
            table = {}
        return table

    def update(self):
        with open(mirror_table, 'w') as f:
            pickle.dump(self.table, f)


def fetch_js(url):
    """Fetches the link to the js file that hopefully contains the video url"""
    soup = get_soup(url)
    try:
        js = soup.select('#embed_holder > iframe')[0].get('src', None)
    except:
        js = None
    return js


def get_vidname(url):
    """
    gets episode name number string
    """
    soup = get_soup(url)
    try:
        name = soup.select('#content_head > a:nth-of-type(3)')[0].text
    except:
        name = None
    return name

def get_next_episode_url(url):
    """
    gets the link to the next episode.
    """
    soup=get_soup(url)
    nextlink=None
    try:
        div=soup.findAll('div',style="FONT-WEIGHT: bold; FONT-SIZE: 12px; PADDING-TOP: 3px")[-1]
        anchor=div.find('a')
        if 'NEXT' in anchor.text:
            nextlink=anchor.get('href',None)
    except:
        nextlink=None
    return nextlink

def main():
    if not exists(allanimes):
        make_pickle()
    animes = get_animes_sample()
    t = Table()
    for anime, url in animes:
        episode = get_random_episode(url)
        if episode:
            print '\n', episode
            mirrors = get_mirrors(episode)
            if mirrors:
                for name, murl in mirrors:
                    if name not in t.table.keys():
                        js = fetch_js(murl)
                        if js and all(rejectable not in js for rejectable in reject_list):
                            t.table[name] = js
                            print name, '   ', t.table[name]

    t.update()


if __name__ == '__main__':
    main()



# coding: utf-8
import pickle
table=pickle.load(open('mirror_table.p'))
html='''<ol>{}</ol>'''.format('\n'.join(['''<li><a href="{}">{}</a></li>'''.format(v,k) for k,v in table.items()]))
open('table.html','w').write(html)




from setuptools import setup
import os
def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()
setup(name='nwanime_dl',
      version='1.8.2',
      description='downloads anime from nwanime',
      long_description=(read('readme.rst')),
      url='https://github.com/thekindlyone/nwanime-dl',
      author='thekindlyone',
      author_email='dodo.dodder@gmail.com',
      license='GNU GPL v2',
      packages=['nwanime_dl'],
      install_requires=[
          'beautifulsoup4',
          'requests'   ],
      scripts=['bin/nwanime-dl'],
      classifiers=[
          "Development Status :: 4 - Beta",
          "Topic :: Utilities",
          "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
          "Operating System :: POSIX :: Linux",
          "Programming Language :: Python :: 2.7"          
      ],
      zip_safe=False)


#!/usr/bin/python
# coding: utf-8
import requests
from bs4 import BeautifulSoup
import sys
import subprocess
import re
import regexes
from helper import get_soup, get_mirrors, fetch_js, get_vidname
from urlparse import urlparse
from os.path import splitext


def get_ext(video_url):
    """get extention of file from video url"""
    components = urlparse(video_url)
    return splitext(components.path)[-1]



def download(url):
    """
    Downloads video in url. 
    download(url)
    """
    vidname = get_vidname(url)
    print vidname
    print 'fetching mirrors'
    mirrors = get_mirrors(url)
    print 'mirrors found'
    for mirror, link in mirrors:
        if mirror in regexes.extractors:
            print 'downloadable mirror found'
            jsfile = fetch_js(link)
            if jsfile:
                r = requests.get(jsfile)
                if r.status_code == 200:
                    video = re.search(regexes.extractors[mirror], r.content)
                    if video:
                        video = video.group(1)
                        outfile = vidname + get_ext(video)
                        print 'attempting download from', mirror
                        wget = subprocess.Popen(['wget','-O', outfile, video])
                        wget.wait()
                        return wget.returncode,vidname
    else:
        print "Can't Download. No downloadable mirrors found. Sorry."
        return 1,vidname

def main():
    if len(sys.argv) > 1:
        url = sys.argv[1]
        download(url)
    else:
        print 'usage :nwanime-dl <url>'
        sys.exit(1)



if __name__ == '__main__':
    main()